﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

namespace GoniometricFunctions
{
    class Plotter
    {
        private readonly int marginX = 20;
        private readonly int marginY = 10;
        private readonly int sstepsX = 4;
        private readonly int sstepsY = 10;

        private Control canvas;

        private int zeroX;
        private int zeroY;
        private int width;
        private int height;

        private double stepX;
        private double stepY;

        private AxisDefinition xAxis;
        private AxisDefinition yAxis;

        private bool _HasAxes = false;
        public bool HasAxes { get { return _HasAxes; } }

        public Plotter(Control canvas)
        {
            width = canvas.Width - (2 * marginX);
            height = canvas.Height - (2 * marginY);
            this.canvas = canvas;
        }

        public void Erase()
        {
            _HasAxes = false;
            canvas.Refresh();
        }

        public void SetAxes(AxisDefinition x, AxisDefinition y)
        {
            _HasAxes = true;
            xAxis = x;
            yAxis = y;

            stepX = (double)(x.Spread * 2) / (double)width;
            stepY = (double)(y.Spread * 2) / (double)height;

            zeroX = marginX + (width / 2);
            zeroY = marginY + (height / 2);

            
                using (Graphics g = canvas.CreateGraphics())
                {
                    g.Clear(canvas.BackColor);
                    using (Pen p = new Pen(Color.Black))
                    {
                        using (Brush b = new SolidBrush(Color.Black))
                        {
                            // draw X axis
                            g.DrawLine(p, marginX, zeroY, marginX + width, zeroY);

                            // draw Y axis
                            g.DrawLine(p, zeroX, marginY, zeroX, marginY + height);

                            DrawAxisMarks(xAxis, false, stepX, sstepsX, g, p, b);
                            DrawAxisMarks(yAxis, true, stepY, sstepsY, g, p, b);

                        }
                    }
                   
            }

        }

        private void DrawAxisMarks(AxisDefinition axis, bool vertical, double step, int ssteps, Graphics g, Pen p, Brush b)
        {

            PointF pointStart;
            PointF pointEnd;
            PointF pointText;
            float x;
            float y;
            int corrX;
            int corrY;
            int txtCorrX;
            int txtCorrY;

            double val = -1 * axis.Spread;
            int steps = 0;


            while (axis.Spread - val > -1 * (1 / (double)ssteps))
            {
                
                if (!vertical)
                {
                    x = (float)(zeroX + (val / step));
                    y = zeroY;
                }
                else
                {
                    x = zeroX;
                    y = (float)(zeroY - (val / step));
                }
                
                // line
                
                if ((steps % ssteps) == 0 && !vertical)
                {
                    corrX = 0;
                    corrY = 10;
                    txtCorrX = -5;
                    txtCorrY = 20;
                }
                else if ((steps % ssteps) == 0 && vertical)
                {
                    corrX = 10;
                    corrY = 0;
                    txtCorrX = 20;
                    txtCorrY = -5;
                }
                else if (!vertical)
                {
                    corrX = 0;
                    corrY = 5;
                    txtCorrX = -5;
                    txtCorrY = 10;
                }
                else
                {
                    corrX = 5;
                    corrY = 0;
                    txtCorrX = 10;
                    txtCorrY = -5;
                }

                pointStart = new PointF(x - corrX, y - corrY);
                pointEnd = new PointF(x + corrX, y + corrY);
                pointText = new PointF(x + txtCorrX, y + txtCorrY);

                if (axis.ShowUnitMarks && (steps % ssteps) == 0)
                {
                    g.DrawLine(p, pointStart, pointEnd);
                }
                if (axis.ShowUnitTexts && (steps % ssteps) == 0)
                {
                    g.DrawString(String.Format("{0:#}", val), canvas.Font, b, pointText);
                }
                // line
                if (axis.ShowSmallMarks)
                {
                    g.DrawLine(p, pointStart, pointEnd);
                }
                // text
                if (axis.ShowSmallTexts)
                {
                    g.DrawString(String.Format("{0:0.#}", val), canvas.Font, b, pointText);
                }

                if (axis.ShowSmallMarks || axis.ShowSmallTexts)
                {
                    val += 1 / (double)ssteps;
                    steps += 1;
                }
                else
                {
                    val += 1;
                    steps += ssteps;
                }
            }
        }

        public void DrawFunction(IFunction f, Color c)
        {
            
            
                using (Graphics g = canvas.CreateGraphics())
                {
                    
                    using (Pen p = new Pen(c))
                    {
                        double xVal = -1 * xAxis.Spread;
                        double yPosS = 0;
                        double yPosF = f.GetFuncValue(xVal) / stepY;

                        int posX = 1;
                        xVal += stepX;
                        while (posX < width)
                        {
                            yPosS = yPosF;
                            yPosF = f.GetFuncValue(xVal) / stepY;
                            if(Math.Abs(yPosS) < (height / 2) && Math.Abs(yPosF) < (height / 2))
                                g.DrawLine(p, marginX + posX - 1, zeroY - (float)yPosS, marginX + posX, zeroY - (float)yPosF);

                            posX += 1;
                            xVal += stepX;
                        }
                    }
                }
            

        }
    }
}
